// main.js
const { app, BrowserWindow, Tray, Menu, nativeImage, ipcMain, shell } = require('electron');
const path = require('path');
const fs = require('fs');

let mainWindow = null;
let tray = null;
let scanWindows = [];

// Determine if the app is packaged
const isPackaged = app.isPackaged;

// Define base path based on the environment
const basePath = isPackaged ? path.join(process.resourcesPath, 'app') : app.getAppPath();

// Add these lines at the beginning of main.js after defining basePath
console.log('App is packaged:', isPackaged);
console.log('Base Path:', basePath);
console.log('Base HTML Path:', path.join(basePath, 'dist', 'base.html'));
console.log('Scan Window HTML Path:', path.join(basePath, 'dist', 'scanWindow.html'));

// Access appicon.png outside ASAR
const appiconPath = path.join(basePath, 'assets', 'icons', 'appicon.png');

// Define paths for fingerprints.json and modified_fingerprints.json
const defaultFilePath = path.join(basePath, 'fingerprints.json');
const modifiedFilePath = path.join(app.getPath('userData'), 'modified_fingerprints.json');

function createWindow() {
  mainWindow = new BrowserWindow({
    width: 900,
    height: 750,
    resizable: true,
    minWidth: 900,
    minHeight: 750,
    maximizable: true,
    fullscreenable: false,
    webPreferences: {
      preload: path.join(basePath, 'src', 'main', 'preload.js'), 
      nodeIntegration: true,
      contextIsolation: false,
    },
  });
  Menu.setApplicationMenu(null);

  // Load base.html from the bundled resources
  mainWindow.loadFile(path.join(basePath, 'dist', 'base.html'));

  // Prevent the app from quitting when the window is closed
  mainWindow.on('close', (event) => {
    if (!app.isQuitting) {
      event.preventDefault();
      mainWindow.hide();
      if (tray) {
        tray.displayBalloon({
          icon: appiconPath,
          title: app.getName(),
          content:
            'Your app is still running in the background. Click the tray icon to show.',
        });
      }
    }
  });
}

app.whenReady().then(() => {
  createWindow();

  const iconPath = appiconPath;
  tray = new Tray(nativeImage.createFromPath(iconPath));
  const contextMenu = Menu.buildFromTemplate([
    {
      label: 'Show App',
      click: () => {
        mainWindow.show();
      },
    },
    {
      label: 'Quit',
      click: () => {
        app.isQuitting = true;
        app.quit();
      },
    },
  ]);

  tray.setToolTip(app.getName());
  tray.setContextMenu(contextMenu);

  tray.on('click', () => {
    mainWindow.show();
  });

  app.on('activate', () => {
    if (BrowserWindow.getAllWindows().length === 0) createWindow();
  });
});

// IPC function for getting app version
ipcMain.handle('get-version', () => {
  return app.getVersion();
});

// IPC functions for loading, saving, and resetting fingerprints
ipcMain.handle('load-fingerprints', async () => {
  let fingerprints = [];
  let lastModified = null;

  if (fs.existsSync(modifiedFilePath)) {
    fingerprints = JSON.parse(fs.readFileSync(modifiedFilePath, 'utf-8'));
    lastModified = fs.statSync(modifiedFilePath).mtime;
  } else if (fs.existsSync(defaultFilePath)) {
    fingerprints = JSON.parse(fs.readFileSync(defaultFilePath, 'utf-8'));
  }

  return { fingerprints, lastModified };
});

ipcMain.on('save-fingerprints', (event, fingerprints) => {
  fs.writeFileSync(modifiedFilePath, JSON.stringify(fingerprints, null, 2));
});

ipcMain.handle('reset-fingerprints', async () => {
  if (fs.existsSync(modifiedFilePath)) {
    fs.unlinkSync(modifiedFilePath);
  }

  if (fs.existsSync(defaultFilePath)) {
    const fingerprints = JSON.parse(fs.readFileSync(defaultFilePath, 'utf-8'));
    return { fingerprints };
  } else {
    return { fingerprints: [] };
  }
});

// Adjusted path for takeoverScanner.js using __dirname
const takeoverScannerPath = path.join(__dirname, '..', 'takeoverScanner.js');
const { scanTakeover } = require(takeoverScannerPath);

const scanFiles = new Map(); // Map to handle multiple file writing operations at once

// IPC handler to open a new window for performing subdomain takeover scans 
ipcMain.on('open-scan-window', (event, scanData) => {
  const newScanWindow = new BrowserWindow({
    width: 800,
    height: 600,
    minWidth: 800,
    minHeight: 600,
    resizable: true,
    fullscreenable: false,
    webPreferences: {
      preload: path.join(basePath, 'src', 'main', 'preload.js'), // Updated path
      nodeIntegration: true,
      contextIsolation: false,    // Don't worry about this
      sandbox: false
    }
  });

  const query = new URLSearchParams({
    scanName: scanData.scanName || '',
    fileName: scanData.fileName || '',
    timeout: scanData.timeout.toString(),
    threads: scanData.threads.toString(),
    useLiveSiteScanner: scanData.useLiveSiteScanner ? '1' : '0'
  });

  const scanWindowHTMLPath = path.join(basePath, 'dist', 'scanWindow.html');
  const fileUrl = new URL(`file://${scanWindowHTMLPath}`);
  fileUrl.search = query.toString();

  newScanWindow.loadURL(fileUrl.toString());

  // Add the new window to the array
  scanWindows.push(newScanWindow);

  const stoppedRef = { current: false };

  // Handle file operations if save option is enabled
  if (scanData.fileName) {
    const scansDir = path.join(app.getPath('userData'), 'takeover_scans');

    // Create 'takeover_scans' directory if it doesn't exist
    if (!fs.existsSync(scansDir)) {
      fs.mkdirSync(scansDir, { recursive: true });
    }

    const filePath = path.join(scansDir, scanData.fileName);

    // Initialize scanInfo with write stream and current ID
    const scanInfo = {
      stream: fs.createWriteStream(filePath, { flags: 'w' }),
      currentId: 1,
      isFirstEntry: true // To handle commas
    };

    // Handle write stream errors
    scanInfo.stream.on('error', (err) => {
      console.error(`Error writing to file ${scanData.fileName}:`, err);
      newScanWindow.webContents.send('scan-error', { message: `Failed to write to file: ${err.message}` });
    });

    // Store the scanInfo in the map with the window's ID as the key
    scanFiles.set(newScanWindow.id, scanInfo);
  }

  // Start takeover scanning after the window finishes loading
  newScanWindow.webContents.on('did-finish-load', () => {
    scanTakeover(
      scanData.targets,
      scanData.timeout,
      scanData.threads,
      scanData.useLiveSiteScanner,
      (completed, totalCount) => {
        // Update progress in scan window
        newScanWindow.webContents.send('scan-progress', { completed, totalCount });
      },
      (id, service, url, status, timestamp) => {
        // Send result row to scan window
        newScanWindow.webContents.send('scan-result', { id, service, url, status, timestamp });

        // Handle file writing if saving is enabled
        if (scanData.fileName) {
          const scanInfo = scanFiles.get(newScanWindow.id);
          if (scanInfo) {
            const { stream, currentId, isFirstEntry } = scanInfo;

            // JSON format for saving data:
            const entry = {
              service_name: service,
              timestamp: timestamp,
              url: url,
              status: status
            };

            const serializedEntry = `  "${currentId}": ${JSON.stringify(entry, null, 4)}`;
            const entryWithComma = isFirstEntry ? `${serializedEntry}\n` : `,${serializedEntry}\n`;

            stream.write(entryWithComma, (err) => {
              if (err) {
                console.error(`Error writing to file ${scanData.fileName}:`, err);
                newScanWindow.webContents.send('scan-error', { message: `Failed to write to file: ${err.message}` });
              }
            });

            // Update scanInfo for the next JSON entry
            scanInfo.currentId += 1;
            scanInfo.isFirstEntry = false;
          }
        }
      },
      stoppedRef
    );
  });

  // Handle closing of the scan window
  newScanWindow.on('close', (event) => {
    if (scanFiles.has(newScanWindow.id)) {
      event.preventDefault();

      const scanInfo = scanFiles.get(newScanWindow.id);
      if (scanInfo) {
        scanInfo.stream.end('}\n', (err) => {
          if (err) {
            console.error(`Error closing file ${scanData.fileName}:`, err);
            newScanWindow.webContents.send('scan-error', { message: `Failed to close file: ${err.message}` });
          } else {
            console.log(`Successfully closed file ${scanData.fileName}`);
          }
          scanFiles.delete(newScanWindow.id);
          newScanWindow.destroy();
        });
      } else {
        newScanWindow.destroy();
      }
    } else {
      // If saving is not enabled, allow the window to close normally
      newScanWindow.destroy();
    }
  });
});


// Function to get the list of files in the 'takeover_scans' directory
ipcMain.handle('get-takeover-reports', async () => {
  const scansDir = path.join(app.getPath('userData'), 'takeover_scans');

  if (!fs.existsSync(scansDir)) {
    return [];
  }

  const files = fs.readdirSync(scansDir);
  return files.map((file, index) => {
    const filePath = path.join(scansDir, file);
    const stats = fs.statSync(filePath);
    const creationDate = new Date(stats.birthtime).toLocaleString('en-US', {
      hour: '2-digit',
      minute: '2-digit',
      day: 'numeric',
      month: 'long',
      year: 'numeric',
    });
    return {
      id: index + 1,
      name: file,
      creationDate,
      path: filePath,
    };
  });
});


// Function to use the system's "Open With" dialog for opening a file
ipcMain.handle('open-file', async (event, filePath) => {
  const platform = process.platform; // Determine the OS

  try {
    if (platform === 'win32' || platform === 'linux') {
      // Use the default "Open With" dialog
      shell.openPath(filePath); // Opens file with the default application or triggers "Open With"
    } else if (platform === 'darwin') {
      // macOS: Use 'open' command to invoke the system "Open With" dialog
      const exec = require('child_process').exec;
      exec(`open -a Finder ${filePath}`, (err) => {
        if (err) {
          console.error('Error opening file on macOS:', err);
        }
      });
    } else {
      throw new Error(`Unsupported platform: ${platform}`);
    }
  } catch (err) {
    console.error('Error opening file:', err);
    event.reply('error', `Could not open file: ${err.message}`);
  }
});


app.on('before-quit', () => (app.isQuitting = true));
