import React, { useState, useEffect } from 'react';
const { ipcRenderer } = window.require('electron');
const { shell } = window.require('electron');

function About() {
  const [activeTab, setActiveTab] = useState('update');
  const [currentVersion, setCurrentVersion] = useState('Loading...');
  const [latestVersion, setLatestVersion] = useState(null);
  const [releaseDate, setReleaseDate] = useState(null);
  const [isLatest, setIsLatest] = useState(false);
  const [isChecking, setIsChecking] = useState(false);
  const [error, setError] = useState('');

  // Handler to switch tabs
  const handleTabClick = (tab) => {
    setActiveTab(tab);
    setError('');
  };

  // Fetch current app version on component mount
  useEffect(() => {
    ipcRenderer
      .invoke('get-version')
      .then((version) => {
        setCurrentVersion(version);
      })
      .catch(() => {
        setCurrentVersion('Unknown');
        setError('Unable to retrieve app version.');
        setTimeout(() => {
          setError('');
        }, 5000);
      });
  }, []);

  // Function to check for updates
  const checkForUpdates = () => {
    setIsChecking(true);
    setError('');
    const updateUrl = 'https://0xh.ca/open-source/subwatch/version';      // URL for getting app's latest version

    // Create an AbortController to handle timeout
    const controller = new AbortController();
    const timeoutId = setTimeout(() => {
      controller.abort();
    }, 20000);

    fetch(updateUrl, { signal: controller.signal })
      .then((response) => {
        clearTimeout(timeoutId);
        if (!response.ok) {
          throw new Error(`HTTP error! Status: ${response.status}`);
        }
        return response.json(); 
      })
      .then((data) => {
        if (data.version && data.release_date) {
          setLatestVersion(data.version);
          setReleaseDate(data.release_date);
          // Compare versions
          if (currentVersion === data.version) {
            setIsLatest(true);
          } else {
            setIsLatest(false);
          }
        } else {
          throw new Error('Invalid response format');
        }
      })
      .catch((err) => {
        if (err.name === 'AbortError') {
          setError('Update check timed out. Please try again later.');
        } else {
          setError('Failed to check for updates. Please try again later.');
        }

        setTimeout(() => {
          setError('');
        }, 5000);
      })
      .finally(() => {
        setIsChecking(false);
      });
  };

  // Function to open the link in the default browser
  const openExternalLink = (url) => {
    shell.openExternal(url);
  };

  return (
    <div className="container mt-4">
      <h1>About Page</h1>

      {/* Navigation tabs inside About page */}
      <ul className="nav nav-tabs mt-4" id="aboutTab" role="tablist">
        <li className="nav-item" role="presentation">
          <button
            className={`nav-link ${activeTab === 'update' ? 'active' : ''}`}
            id="update-tab"
            data-bs-toggle="tab"
            type="button"
            role="tab"
            aria-controls="update"
            aria-selected={activeTab === 'update'}
            onClick={() => handleTabClick('update')}
          >
            Update
          </button>
        </li>
        <li className="nav-item" role="presentation">
          <button
            className={`nav-link ${activeTab === 'support' ? 'active' : ''}`}
            id="support-tab"
            data-bs-toggle="tab"
            type="button"
            role="tab"
            aria-controls="support"
            aria-selected={activeTab === 'support'}
            onClick={() => handleTabClick('support')}
          >
            Support
          </button>
        </li>
        <li className="nav-item" role="presentation">
          <button
            className={`nav-link ${activeTab === 'tos' ? 'active' : ''}`}
            id="tos-tab"
            data-bs-toggle="tab"
            type="button"
            role="tab"
            aria-controls="tos"
            aria-selected={activeTab === 'tos'}
            onClick={() => handleTabClick('tos')}
          >
            ToS/Legal
          </button>
        </li>
      </ul>

      {/* Tab content */}
      <div className="tab-content mt-3" id="aboutTabContent">
        {/* Update Tab Pane */}
        <div
          className={`tab-pane fade ${activeTab === 'update' ? 'show active' : ''}`}
          id="update"
          role="tabpanel"
          aria-labelledby="update-tab"
        >
          <h3>Version</h3>
          <p>
            Current Version: <strong>{currentVersion}</strong>
          </p>
          {error && (
            <div className="alert alert-danger" role="alert">
              {error}
            </div>
          )}
          {/*
          {latestVersion ? (
            isLatest ? (
              <div className="alert alert-success" role="alert">
                You are using the latest version.
              </div>
            ) : (
              
              <div>
                <button
                  className="btn btn-primary"
                  onClick={() => openExternalLink('pornhub.gc.ca')}
                >
                  Download New Version
                </button>
                <p className="mt-2">New version {latestVersion} released on {releaseDate}.</p>
              </div>
              
            )
          ) : (
            <button className="btn btn-primary" onClick={checkForUpdates} disabled={isChecking}>
              {isChecking ? (
                <>
                  <span className="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>
                  {' '}Checking...
                </>
              ) : (
                'Check Update'
              )}
            </button>
          )}
            */}
        </div>

        {/* Support Tab Pane */}
        <div
          className={`tab-pane fade ${activeTab === 'support' ? 'show active' : ''}`}
          id="support"
          role="tabpanel"
          aria-labelledby="support-tab"
        >
          <h3>Support</h3>
          <p>
            Need help? Reach out to me at{' '}
            <a href="mailto:0x.hamy.1@gmail.com">0x.hamy.1@gmail.com</a>.
          </p>
        </div>

        {/* ToS/Legal Tab Pane */}
        <div
          className={`tab-pane fade ${activeTab === 'tos' ? 'show active' : ''}`}
          id="tos"
          role="tabpanel"
          aria-labelledby="tos-tab"
        >
          <h3>Terms of Service & Legal Disclaimer</h3>
          <br /><br />
          <h4>Terms of Service</h4>
          <p>
            By using SubWatch (the "App"), you agree to comply with and be bound by the following terms and conditions. If you do not agree with these terms, you may not use the App.
          </p>
          <ul>
            <li>
              <strong>Authorized Use:</strong> The App is intended solely for lawful purposes, including monitoring and securing your own subdomains or those for which you have explicit authorization. Unauthorized use of the App to target systems without proper permission is strictly prohibited and may violate applicable laws.
            </li>
            <li>
              <strong>No Data Collection:</strong> SubWatch does not collect or store any personal or usage data. All inputs and outputs remain local to your device.
            </li>
            <li>
              <strong>Intellectual Property:</strong> The App, including its code, design, and features, is protected by intellectual property laws. Unauthorized copying, distribution, or modification is prohibited.
            </li>
            <li>
              <strong>Updates and Support:</strong> SubWatch is provided "as is," and we are not obligated to provide updates or support. Any updates, if provided, are subject to these terms.
            </li>
            <li>
              <strong>Limitation of Liability:</strong> To the fullest extent permitted by law, the developers of SubWatch shall not be held liable for any damages resulting from the use or misuse of the App.
            </li>
          </ul>

          <br />
          <h4>Legal Disclaimer</h4>
          <p>
            SubWatch is a tool designed to assist users in monitoring and securing their digital assets, specifically subdomains. It is your responsibility to ensure that your use of this tool complies with all applicable laws and regulations in your jurisdiction. Unauthorized scanning or exploitation of systems without explicit permission may constitute a violation of law and could result in civil or criminal penalties.
          </p>
          <p>
            By using SubWatch, you acknowledge and agree that:
          </p>
          <ul>
            <li>
              The developers and distributors of SubWatch are not responsible for any unauthorized or illegal activity conducted using this tool.
            </li>
            <li>
              You bear full responsibility for ensuring that you have the necessary authorization to perform scans on the systems you target.
            </li>
            <li>
              The tool is provided "as is" without any warranties, express or implied, including but not limited to fitness for a particular purpose or non-infringement.
            </li>
            <li>
              Any misuse of this tool that results in harm, loss, or legal consequences is solely your responsibility.
            </li>
          </ul>

          <p>
            For additional information or support, please refer to the "About" section in the app or contact us through the provided support channels.
          </p>
        </div>
      </div>
    </div>
  );
}

export default About;
