import React, { useState, useEffect } from "react";
const { ipcRenderer } = window.require("electron");


const Fingerprints = () => {
  const [fingerprints, setFingerprints] = useState({});
  const [serviceName, setServiceName] = useState("");
  const [fingerprintText, setFingerprintText] = useState("");
  const [lastModified, setLastModified] = useState(null);
  // Editing state
  const [editId, setEditId] = useState(null);
  const [editServiceName, setEditServiceName] = useState("");
  const [editLines, setEditLines] = useState([]); 
  const [originalServiceName, setOriginalServiceName] = useState("");
  const [originalLines, setOriginalLines] = useState([]);

  useEffect(() => {
    ipcRenderer.invoke("load-fingerprints").then((data) => {
      setFingerprints(data.fingerprints);
      setLastModified(data.lastModified);
    });
  }, []);


  const parseFingerprintInput = (text) => {
    const lines = text.split("\n").map((line) => line.trim()).filter(Boolean);
    return lines.length > 1 ? lines : lines[0] || "";
  };


  const refreshFingerprints = () => {
    // After any save operation, reload to get fresh fingerprints and its last modification date 
    ipcRenderer.invoke("load-fingerprints").then((data) => {
      setFingerprints(data.fingerprints);
      setLastModified(data.lastModified);
      cancelEdit(); 
    });
  };


  const handleAddFingerprint = () => {
    if (!serviceName || !fingerprintText.trim()) {
      alert("Please fill out both fields.");
      return;
    }

    const newId = (Object.keys(fingerprints).length + 1).toString();
    const updatedFingerprints = {
      ...fingerprints,
      [newId]: {
        service_name: serviceName,
        fingerprint: parseFingerprintInput(fingerprintText.trim()),
      },
    };

    ipcRenderer.send("save-fingerprints", updatedFingerprints);
    refreshFingerprints();

    setServiceName("");
    setFingerprintText("");
  };


  const handleDeleteFingerprint = (id) => {
    const updatedFingerprints = { ...fingerprints };
    delete updatedFingerprints[id];

    const reassignedFingerprints = {};
    Object.values(updatedFingerprints).forEach((fp, index) => {
      reassignedFingerprints[(index + 1).toString()] = fp;
    });

    ipcRenderer.send("save-fingerprints", reassignedFingerprints);
    refreshFingerprints();
  };


  const handleResetFingerprints = () => {
    ipcRenderer.invoke("reset-fingerprints").then((data) => {
      setFingerprints(data.fingerprints);
      setLastModified(null);
      cancelEdit();
    });
  };


  const handleEditFingerprint = (id) => {
    const { service_name, fingerprint } = fingerprints[id];
    setEditId(id);
    setEditServiceName(service_name);
  
    const lines = Array.isArray(fingerprint) ? fingerprint : [fingerprint];
    setEditLines(lines);
    setOriginalServiceName(service_name);
    setOriginalLines(lines);
  };
  

  const cancelEdit = () => {
    setEditId(null);
    setEditServiceName("");
    setEditLines([]);
  };

  
  const saveEdit = (id) => {
    const finalLines = editLines.map(line => line.trim()).filter(Boolean);
  
    if (finalLines.length === 0) {
      alert("Please provide at least one non-empty fingerprint line.");
      return;
    }
  
    let finalFingerprint;
    if (finalLines.length > 1) {
      finalFingerprint = finalLines;
    } else {
      finalFingerprint = finalLines[0] || "";
    }
  
    const trimmedServiceName = editServiceName.trim();
    const originalFingerprintMultiple = originalLines.length > 1 ? originalLines : originalLines[0] || "";
    const noChangeInServiceName = trimmedServiceName === originalServiceName.trim();
    const noChangeInFingerprint = JSON.stringify(finalFingerprint) === JSON.stringify(originalFingerprintMultiple);
  
    if (noChangeInServiceName && noChangeInFingerprint) {
      cancelEdit();
      return;
    }
  
    const updatedFingerprints = { ...fingerprints };
    updatedFingerprints[id] = {
      service_name: trimmedServiceName,
      fingerprint: finalFingerprint,
    };
  
    ipcRenderer.send("save-fingerprints", updatedFingerprints);
    refreshFingerprints();
  };
  


  // Helper function to render fingerprints in view mode with line numbering and highlighting
  const renderFingerprint = (fingerprint) => {
    if (Array.isArray(fingerprint)) {
      return fingerprint.map((line, index) => (
        <pre
          key={index}
          style={{
            background: "#f8f9fa",
            margin: 0,
            padding: "5px",
            borderBottom: "1px solid #eee"
          }}
        >
          <code>{`${index + 1}: ${line}`}</code>
        </pre>
      ));
    } else {
      // Single line fingerprint
      return (
        <pre
          style={{
            background: "#f8f9fa",
            margin: 0,
            padding: "5px"
          }}
        >
          <code>{fingerprint}</code>
        </pre>
      );
    }
  };

  // Render edit mode: multiple input lines, each line editable, line numbered, all highlighted
  const renderEditMode = () => {
    return (
      <code style={{ 
        display: "block", 
        border: "1px solid #ddd", 
        background: "#f8f9fa", 
        padding: "5px"
      }}>
        <div style={{display: "flex", flexDirection: "column", gap: "5px"}}>
          {editLines.map((line, i) => (
            <div key={i} style={{display: "flex", alignItems: "center"}}>
              <span style={{
                display: "inline-block",
                width: "30px",
                textAlign: "right",
                marginRight: "10px"
              }}>
                <code style={{ background: "none", padding: "0 5px" }}>{i + 1}</code>
              </span>
              <input
                type="text"
                className="form-control"
                style={{
                  fontFamily: "monospace",
                  fontSize: "14px",
                  lineHeight: "1.5",
                }}
                value={line}
                onChange={(e) => {
                  const newLines = [...editLines];
                  newLines[i] = e.target.value;
                  setEditLines(newLines);
                }}
              />
              <button
                type="button"
                className="btn btn-sm btn-danger ms-2"
                onClick={() => {
                  const newLines = [...editLines];
                  newLines.splice(i, 1);
                  setEditLines(newLines);
                }}
              >
                Remove
              </button>
            </div>
          ))}
          <button
            type="button"
            className="btn btn-sm btn-secondary mt-2"
            onClick={() => setEditLines([...editLines, ""])}
          >
            Add Line
          </button>
        </div>
      </code>
    );
  };

  return (
    <div className="container my-4">
      <h1 className="text-center mb-4">Fingerprints</h1>
      {lastModified && (
        <p className="text-muted text-center">
          Last modified: {new Date(lastModified).toLocaleString()}
        </p>
      )}
      <div className="card mb-4">
        <div className="card-header">Add New Fingerprint</div>
        <div className="card-body">
          <form
            onSubmit={(e) => {
              e.preventDefault();
              handleAddFingerprint();
            }}
          >
            <div className="mb-3">
              <label htmlFor="serviceName" className="form-label">
                Service Name
              </label>
              <input
                type="text"
                id="serviceName"
                className="form-control"
                value={serviceName}
                onChange={(e) => setServiceName(e.target.value)}
                required
              />
            </div>
            <div className="mb-3">
              <label htmlFor="fingerprintText" className="form-label">
                Fingerprint (one per line)
              </label>
              <textarea
                id="fingerprintText"
                className="form-control"
                style={{
                  fontFamily: "monospace",
                  fontSize: "14px",
                  lineHeight: "1.5",
                  resize: "none",
                  height: "150px",
                }}
                rows="3"
                value={fingerprintText}
                onChange={(e) => setFingerprintText(e.target.value)}
                required
              ></textarea>
            </div>
            <button type="submit" className="btn btn-primary">
              Add Fingerprint
            </button>
          </form>
        </div>
      </div>
      {lastModified && (
        <div className="mb-3">
          <button className="btn btn-danger" onClick={handleResetFingerprints}>
            Reset Fingerprints
          </button>
          <small className="text-muted d-block mt-2">
            Resetting will delete the modified fingerprints file and restore
            the default fingerprints.
          </small>
        </div>
      )}
      <div className="table-responsive">
        <table className="table table-striped table-hover">
          <thead>
            <tr>
              <th>ID</th>
              <th>Service Name</th>
              <th>Fingerprint</th>
              <th>Actions</th>
            </tr>
          </thead>
          <tbody>
            {Object.entries(fingerprints).map(([id, { service_name, fingerprint }]) => {
              const isEditing = editId === id;
              return (
                <tr key={id}>
                  <td>{id}</td>
                  <td>
                    {isEditing ? (
                      <input
                        type="text"
                        className="form-control"
                        value={editServiceName}
                        onChange={(e) => setEditServiceName(e.target.value)}
                      />
                    ) : (
                      service_name
                    )}
                  </td>
                  <td style={{ verticalAlign: "top" }}>
                    {isEditing ? (
                      renderEditMode()
                    ) : (
                      <div
                        style={{
                          maxWidth: "400px",
                          overflowX: "auto",
                          whiteSpace: "pre",
                          border: "1px solid #ddd"
                        }}
                      >
                        {renderFingerprint(fingerprint)}
                      </div>
                    )}
                  </td>
                  <td style={{ verticalAlign: "top" }}>
                    {isEditing ? (
                      <>
                        <button
                          className="btn btn-success btn-sm me-2"
                          onClick={() => saveEdit(id)}
                        >
                          Save
                        </button>
                        <button
                          className="btn btn-secondary btn-sm"
                          onClick={cancelEdit}
                        >
                          Cancel
                        </button>
                      </>
                    ) : (
                      <>
                        <button
                          className="btn btn-primary btn-sm me-2"
                          onClick={() => handleEditFingerprint(id)}
                        >
                          Edit
                        </button>
                        <button
                          className="btn btn-danger btn-sm"
                          onClick={() => handleDeleteFingerprint(id)}
                        >
                          Delete
                        </button>
                      </>
                    )}
                  </td>
                </tr>
              );
            })}
          </tbody>
        </table>
      </div>
    </div>
  );
};

export default Fingerprints;
