import React, { useState, useRef, useEffect } from 'react';


function LiveSite() {
  // Params passed by user through UI
  const [domainsText, setDomainsText] = useState('');
  const [outputText, setOutputText] = useState('');
  const [timeout, setTimeoutValue] = useState(30);
  const [threads, setThreadsValue] = useState(5);

  const [scanning, setScanning] = useState(false);
  const [stopped, setStopped] = useState(false);
  const [completedCount, setCompletedCount] = useState(0);
  const [totalCount, setTotalCount] = useState(0);
  const fileInputRef = useRef(null);

  
  // Track whether scanning is stopped
  const stoppedRef = useRef(stopped);

  useEffect(() => {
    stoppedRef.current = stopped;
  }, [stopped]);


  // Listen for events from the scanning logic (in preload.js)
  useEffect(() => {
    const handleProgress = (e) => {
      const { completed, totalCount } = e.detail;
      setCompletedCount(completed);
    };

    const handleLiveSite = (e) => {
      const { site } = e.detail;
      setOutputText((prev) => prev + site + '\n');
    };

    window.addEventListener('scan-progress', handleProgress);
    window.addEventListener('scan-liveSite', handleLiveSite);

    return () => {
      window.removeEventListener('scan-progress', handleProgress);
      window.removeEventListener('scan-liveSite', handleLiveSite);
    };
  }, []);


  const handleDomainsChange = (e) => {
    setDomainsText(e.target.value);
  };


  const lineCount = (text) => {
    const lines = text.split('\n');
    return lines.length;
  };


  const renderLineNumbers = (count) => {
    const numbers = [];
    for (let i = 1; i <= count; i++) {
      numbers.push(<div key={i}>{i}</div>);
    }
    return numbers;
  };


  const handleLoadFileClick = () => {
    if (fileInputRef.current) {
      fileInputRef.current.click();
    }
  };


  const handleFileChange = (e) => {
    const file = e.target.files[0];
    if (!file) return;
    const reader = new FileReader();
    reader.onload = () => {
      setDomainsText(reader.result);
    };
    reader.readAsText(file);
  };


  const handleCopyToClipboard = () => {
    navigator.clipboard.writeText(outputText).catch((err) => {
      console.error('Failed to copy text: ', err);
    });
  };


  const handleReset = () => {
    setDomainsText('');
    setOutputText('');
    setTimeoutValue(30);
    setThreadsValue(5);
    setScanning(false);
    setStopped(false);
    setCompletedCount(0);
    setTotalCount(0);
  };


  const handleBeginScan = async () => {
    if (scanning) return;
    setScanning(true);
    setStopped(false);
    setCompletedCount(0);
    setOutputText('');
    const list = window.electron.processDomains(domainsText);
    setTotalCount(list.length);

    await window.electron.scanDomains(list, timeout, threads);
    setScanning(false);
    setStopped(false);
  };


  const handleStop = () => {
    if (!scanning) return;
    setStopped(true);
  };

  
  const progressPercentage = totalCount > 0 ? (completedCount / totalCount) * 100 : 0;

  return (
    <div className="container-fluid p-4">
      {/* Controls for timeout and threads */}
      <div className="row mb-4">
        <div className="col-md-6 mb-3">
          <label htmlFor="timeoutInput" className="form-label">
            Timeout (s):
          </label>
          <input
            type="number"
            className="form-control"
            id="timeoutInput"
            min="1"
            value={timeout}
            onChange={(e) => setTimeoutValue(parseInt(e.target.value, 10) || 1)}
          />
        </div>
        <div className="col-md-6 mb-3">
          <label htmlFor="threadsInput" className="form-label">
            Threads:
          </label>
          <input
            type="number"
            className="form-control"
            id="threadsInput"
            min="1"
            value={threads}
            onChange={(e) => setThreadsValue(parseInt(e.target.value, 10) || 1)}
          />
        </div>
      </div>

      {/* Domains Section */}
      <div className="card mb-4">
        <div className="card-header d-flex justify-content-between align-items-center">
          <h5 className="mb-0">Domains</h5>
          <div>
            <button className="btn btn-primary me-2" onClick={handleLoadFileClick}>
              Load Data
            </button>
            <input
              type="file"
              style={{ display: 'none' }}
              ref={fileInputRef}
              onChange={handleFileChange}
              accept=".txt"
            />
          </div>
        </div>
        <div className="card-body">
          <div className="d-flex">
            <div
              className="bg-light text-end me-2"
              style={{
                padding: '0.5em',
                borderRight: '1px solid #ccc',
                userSelect: 'none',
                minWidth: '40px',
              }}
            >
              {renderLineNumbers(lineCount(domainsText))}
            </div>
            <textarea
              className="form-control"
              style={{ height: '200px', resize: 'vertical' }}
              value={domainsText}
              onChange={handleDomainsChange}
              placeholder="Paste domains here..."
            />
          </div>
          <div className="mt-3 d-flex">
            <button className="btn btn-success me-2" onClick={handleBeginScan} disabled={scanning}>
              Begin Scan
            </button>
            <button className="btn btn-secondary" onClick={handleReset} disabled={scanning}>
              Reset
            </button>
          </div>
        </div>
      </div>

      {/* Output Section */}
      <div className="card mb-4">
        <div className="card-header d-flex justify-content-between align-items-center">
          <h5 className="mb-0">Output</h5>
          <button
            className="btn btn-outline-secondary"
            onClick={handleCopyToClipboard}
            disabled={!outputText}
          >
            Copy to Clipboard
          </button>
        </div>
        <div className="card-body">
          <div className="d-flex">
            <div
              className="bg-light text-end me-2"
              style={{
                padding: '0.5em',
                borderRight: '1px solid #ccc',
                userSelect: 'none',
                minWidth: '40px',
              }}
            >
              {renderLineNumbers(lineCount(outputText))}
            </div>
            <textarea
              className="form-control"
              style={{ height: '200px', resize: 'vertical', background: '#f5f5f5' }}
              value={outputText}
              readOnly
              placeholder="Output will appear here..."
            />
          </div>
          {scanning && (
            <div className="mt-3">
              <div className="progress">
                <div
                  className="progress-bar progress-bar-striped progress-bar-animated"
                  role="progressbar"
                  style={{ width: `${progressPercentage}%` }}
                  aria-valuenow={progressPercentage}
                  aria-valuemin="0"
                  aria-valuemax="100"
                ></div>
              </div>
              <div className="mt-2">
                Scanned: {completedCount} / {totalCount}
              </div>
            </div>
          )}
          {totalCount > 0 && (
            <div className="mt-3 d-flex">
              <button
                className={`btn ${stopped ? 'btn-secondary' : 'btn-danger'} me-2`}
                onClick={handleStop}
                disabled={!scanning || stopped}
              >
                {stopped ? 'Stopped' : 'Stop'}
              </button>
            </div>
          )}
        </div>
      </div>
    </div>
  );
}

export default LiveSite;
