import React, { useState, useEffect } from 'react';


function ScanWindowApp() {
  // State for scan parameters
  const [scanName, setScanName] = useState('');
  const [fileName, setFileName] = useState('');
  const [timeout, setTimeout] = useState(0);
  const [threads, setThreads] = useState(0);
  const [useLiveSiteScanner, setUseLiveSiteScanner] = useState(false);

  // State for scan results
  const [results, setResults] = useState([]);     // Table columns and JSON format: {id, service, url, status, timestamp}
  const [completed, setCompleted] = useState(0);
  const [total, setTotal] = useState(0);


  useEffect(() => {
    const parseQueryParams = () => {
      const params = new URLSearchParams(window.location.search);
      const scanNameParam = params.get('scanName') || '';
      const fileNameParam = params.get('fileName') || '';
      const timeoutParam = parseInt(params.get('timeout'), 10) || 0;
      const threadsParam = parseInt(params.get('threads'), 10) || 0;
      const useLiveParam = params.get('useLiveSiteScanner') === '1';

      setScanName(scanNameParam);
      setFileName(fileNameParam);
      setTimeout(timeoutParam);
      setThreads(threadsParam);
      setUseLiveSiteScanner(useLiveParam);
    };

    // Call the function to parse parameters on mount
    parseQueryParams();

    // Listen for scan progress
    window.electron.onScanProgress(({ completed, totalCount }) => {
      console.log(`Progress: ${completed} / ${totalCount}`);
      setCompleted(completed);
      setTotal(totalCount);
    });

    // Listen for scan results
    window.electron.onScanResult(({ id, service, url, status, timestamp }) => {
      console.log(`Result Received: ${url} - Status: ${status} - Timestamp: ${timestamp}`);
      setResults(prev => [...prev, { id, service, url, status, timestamp }]);
    });

    return () => {
      window.electron.removeAllListeners('scan-progress');
      window.electron.removeAllListeners('scan-result');
    };
  }, []);

  const handleStop = () => {
    window.close();
  };


  const progressPercentage = total > 0 ? (completed / total) * 100 : 0;


  return (
    <div className="container my-4">
      <h1 className="mb-4">
        {fileName ? `${scanName} - Saving to: ${fileName}` : scanName}
      </h1>
      <p>
        Timeout: {timeout} seconds<br/>
        Threads: {threads}<br/>
        Live Site Scanner: {useLiveSiteScanner ? 'On' : 'Off'}
      </p>

      <div className="progress mb-3" style={{ height: '20px' }}>
        <div
          className="progress-bar progress-bar-striped progress-bar-animated"
          role="progressbar"
          style={{ width: `${progressPercentage}%` }}
          aria-valuenow={progressPercentage}
          aria-valuemin="0"
          aria-valuemax="100"
        ></div>
      </div>
      <p id="scan-count">Scanned: {completed} / {total}</p>

      <button onClick={handleStop} className="btn btn-danger">Stop & Close</button>

      <br /><br />
      <div style={{ maxHeight: '300px', overflowY: 'auto' }}>
        <table className="table table-striped table-hover mb-4">
          <thead>
            <tr>
              <th>ID</th>
              <th>Service</th>
              <th>URL</th>
              <th>Status</th>
              <th>Timestamp</th>
            </tr>
          </thead>
          <tbody>
            {results.map((res, index) => (
              <tr key={index}>
                <td>{res.id}</td>
                <td>{res.service || 'N/A'}</td>
                <td>{res.url}</td>
                <td>{res.status}</td>
                <td>{res.timestamp || '--'}</td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>

    </div>
  );
}

export default ScanWindowApp;
