import React, { useState, useRef } from "react";


function Takeover() {
  const [timeout, setTimeoutValue] = useState(30);
  const [threads, setThreadsValue] = useState(5);
  const [useLiveSiteScanner, setUseLiveSiteScanner] = useState(false);
  const [urlsText, setUrlsText] = useState("");
  const fileInputRef = useRef(null);
  const [saveOption, setSaveOption] = useState("no-save");
  const [scanName, setScanName] = useState("");
  const [errorMessage, setErrorMessage] = useState(""); 

  const lineCount = (text) => (text ? text.split("\n").length : 1);
  const lines = lineCount(urlsText);

  const renderLineNumbers = (count) => {
    const numbers = [];
    for (let i = 1; i <= count; i++) {
      numbers.push(<div key={i}>{i}</div>);
    }
    return numbers;
  };


  const handleLoadFileClick = () => {
    if (fileInputRef.current) fileInputRef.current.click();
  };


  const handleFileChange = (e) => {
    const file = e.target.files[0];
    if (!file) return;
    const reader = new FileReader();
    reader.onload = () => {
      setUrlsText(reader.result);
    };
    reader.readAsText(file);
  };


  const handleStart = () => {
    if (urlsText.trim() === "") {
      setErrorMessage("Please enter at least one URL to start the scan.");
      return;
    } else {
      setErrorMessage("");
    }

    if (saveOption === "save" && !scanName.trim()) {
      alert("Please provide a scan name.");
      return;
    }

    const chosenName = scanName.trim() || "unnamed-scan";
    let fileName = "";
    if (saveOption === "save") {
      const timestamp = Date.now();
      fileName = `${chosenName.replace(/\s+/g, '_')}_${timestamp}.json`;
    }

    const targets = urlsText.split('\n').map(l => l.trim()).filter(Boolean);

    window.electron.openScanWindow({
      scanName: chosenName,
      fileName,
      timeout,
      threads,
      useLiveSiteScanner,
      targets
    });

    // Reset fields when scan is started and a new window is launched 
    setTimeoutValue(30);
    setThreadsValue(5);
    setUseLiveSiteScanner(false);
    setUrlsText("");
    setSaveOption("no-save");
    setScanName("");
  };


  return (
    <div className="container my-4">
      <h1 className="mb-4">Subdomain Takeover Scanner</h1>

      {/* Scan options */}
      <div className="card mb-4">
        <div className="card-header">Scan Options</div>
        <div className="card-body">
          <div className="mb-3">
            <label htmlFor="timeoutInput" className="form-label">Timeout (s):</label>
            <input
              type="number"
              className="form-control"
              id="timeoutInput"
              min="1"
              value={timeout}
              onChange={(e) => setTimeoutValue(parseInt(e.target.value, 10) || 1)}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="threadsInput" className="form-label">Threads:</label>
            <input
              type="number"
              className="form-control"
              id="threadsInput"
              min="1"
              value={threads}
              onChange={(e) => setThreadsValue(parseInt(e.target.value, 10) || 1)}
            />
          </div>
          <div className="form-check mb-3">
            <input
              className="form-check-input"
              type="checkbox"
              id="liveSiteScannerCheck"
              checked={useLiveSiteScanner}
              onChange={(e) => setUseLiveSiteScanner(e.target.checked)}
            />
            <label className="form-check-label" htmlFor="liveSiteScannerCheck">
              Use Live Site Scanner
            </label>
          </div>
        </div>
      </div>

      {/* URLs input */}
      <div className="card mb-4">
        <div className="card-header d-flex justify-content-between align-items-center">
          <h5 className="mb-0">Targets (URLs)</h5>
          <div>
            <button className="btn btn-primary me-2" onClick={handleLoadFileClick}>
              Load Data
            </button>
            <input
              type="file"
              style={{ display: "none" }}
              ref={fileInputRef}
              onChange={handleFileChange}
              accept=".txt"
            />
          </div>
        </div>
        <div className="card-body" style={{ height: '200px', overflow: 'auto', padding: '0' }}>
          <div className="d-flex" style={{ minHeight: '100%' }}>
            <div
              className="bg-light text-end"
              style={{
                padding: '0.5em',
                borderRight: '1px solid #ccc',
                userSelect: 'none',
                minWidth: '40px',
                flexShrink: 0
              }}
            >
              {renderLineNumbers(lines)}
            </div>
            <div style={{ flex: 1, padding: '0.5em' }}>
              <textarea
                className={`form-control ${errorMessage ? 'is-invalid' : ''}`} 
                style={{
                  border: 'none',
                  outline: 'none',
                  resize: 'none',
                  overflow: 'hidden',
                  width: '100%'
                }}
                rows={lines}
                value={urlsText}
                onChange={(e) => setUrlsText(e.target.value)}
                placeholder="Paste URLs here..."
              />
              {errorMessage && (
                <div className="invalid-feedback">
                  {errorMessage}
                </div>
              )}
            </div>
          </div>
        </div>
      </div>

      {/* Save or not save scan */}
      <div className="card mb-4">
        <div className="card-header">Save Scan Options</div>
        <div className="card-body">
          <div className="form-check mb-3">
            <input
              className="form-check-input"
              type="radio"
              name="saveOption"
              id="noSaveOption"
              value="no-save"
              checked={saveOption === "no-save"}
              onChange={() => setSaveOption("no-save")}
            />
            <label className="form-check-label" htmlFor="noSaveOption">
              Don't Save Scan
            </label>
          </div>

          <div className="form-check mb-3">
            <input
              className="form-check-input"
              type="radio"
              name="saveOption"
              id="saveOption"
              value="save"
              checked={saveOption === "save"}
              onChange={() => setSaveOption("save")}
            />
            <label className="form-check-label" htmlFor="saveOption">
              Save Scan
            </label>
          </div>

          {saveOption === "save" && (
            <div className="mb-3">
              <label htmlFor="scanNameInput" className="form-label">
                Scan Name (required)
              </label>
              <input
                type="text"
                className="form-control"
                id="scanNameInput"
                value={scanName}
                onChange={(e) => setScanName(e.target.value)}
                required={saveOption === "save"}
              />
            </div>
          )}
        </div>
      </div>

      {/* Start button */}
      <div className="mb-4">
        <button
          className="btn btn-success"
          onClick={handleStart}
          disabled={urlsText.trim() === ""} 
        >
          Start
        </button>
      </div>
    </div>
  );
}

export default Takeover;
