const { app } = require('electron');
const fs = require('fs');
const path = require('path');
const fetch = require('node-fetch');
const AbortController = require('abort-controller');
const validator = require('validator'); 
const dns = require('dns').promises;


/**
 * Loads fingerprints from modified_fingerprints.json if it exists,
 * otherwise loads from fingerprints.json.
 * 
 * @returns {Array} - Array of fingerprint objects with service_name and fingerprint.
 */
function loadFingerprints() {
    const appFolder = app.getAppPath(); 
    const modifiedFingerprintsPath = path.join(appFolder, 'modified_fingerprints.json');
    const defaultFingerprintsPath = path.join(appFolder, 'fingerprints.json');

    let fingerprintsData = {};

    // Check if modified_fingerprints.json exists and if we can read it 
    if (fs.existsSync(modifiedFingerprintsPath)) {
        try {
            const data = fs.readFileSync(modifiedFingerprintsPath, 'utf-8');
            fingerprintsData = JSON.parse(data);
            console.log('Loaded modified_fingerprints.json');
        } catch (error) {
            console.error('Error reading modified_fingerprints.json:', error);
            console.log('Falling back to fingerprints.json');
        }
    }

    // If modified_fingerprints.json doesn't exist or failed to load, load fingerprints.json
    if (Object.keys(fingerprintsData).length === 0 && fs.existsSync(defaultFingerprintsPath)) {
        try {
            const data = fs.readFileSync(defaultFingerprintsPath, 'utf-8');
            fingerprintsData = JSON.parse(data);
            console.log('Loaded fingerprints.json');
        } catch (error) {
            console.error('Error reading fingerprints.json:', error);
        }
    }

    // If no fingerprints were loaded, log an error and return an empty array
    if (Object.keys(fingerprintsData).length === 0) {
        console.error('No fingerprints file found. Please ensure fingerprints.json exists.');
        return [];
    }

    // Convert fingerprintsData to an array and return a map/dictionary 
    const fingerprintsArray = Object.values(fingerprintsData).flatMap(fp => {
        if (Array.isArray(fp.fingerprint)) {
            return fp.fingerprint.map(f => ({
                service_name: fp.service_name,
                fingerprint: f
            }));
        } else {
            return [{
                service_name: fp.service_name,
                fingerprint: fp.fingerprint
            }];
        }
    });

    return fingerprintsArray;
}


// Helper function to format date as 'YYYY-MM-DD HH:MM:SS'
function getFormattedTimestamp() {
    const now = new Date();
    const year = now.getFullYear();
    const month = String(now.getMonth() + 1).padStart(2, '0');
    const day = String(now.getDate()).padStart(2, '0');
    const hours = String(now.getHours()).padStart(2, '0');
    const minutes = String(now.getMinutes()).padStart(2, '0');
    const seconds = String(now.getSeconds()).padStart(2, '0');
    
    return `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`;
}


/**
 * Standardizes and validates the domain.
 * 
 * @param {string} domain 
 * @returns {string|null}
 */
async function standardizeDomain(domain) {
    let d = domain.trim();
    let url;
    try {
        if (d.match(/^https?:\/\//)) {
            url = new URL(d);
        } else {
            const hasPort = d.includes(':') && !d.includes('//');
            let protocol = 'https://';
            if (hasPort) {
                const port = d.split(':').pop();
                if (port) {
                    protocol = 'http://';
                }
            }
            url = new URL(protocol + d);
        }

        // Ensure the pathname ends with '/'
        if (!url.pathname || url.pathname === '') {
            url.pathname = '/';
        } else if (!url.pathname.endsWith('/')) {
            url.pathname += '/';
        }

        const standardizedURL = url.toString();

        // Validate the standardized URL syntax
        if (!validator.isURL(standardizedURL, { require_protocol: true })) {
            console.log(`Invalid URL syntax detected and skipped: ${domain}`);
            return null;
        }

        // Validate the FQDN (Fully Qualified Domain Name), skip for localhost
        const hostname = url.hostname;
        if (hostname !== 'localhost' && !validator.isFQDN(hostname, { require_tld: true })) {
            console.log(`Invalid FQDN detected and skipped: ${domain}`);
            return null;
        }

        
        // Perform DNS lookup to ensure the domain exists (disable this for local tests)
        try {
            await dns.lookup(hostname);
        } catch (err) {
            console.log(`DNS lookup failed for domain, skipped: ${domain}`);
            return null;
        }
        

        return standardizedURL;
    } catch (err) {
        console.log(`Invalid domain format detected and skipped: ${domain}`);
        return null; // Return null if URL construction fails
    }
}


/**
 * Checks if the site is live by doing a HEAD request.
 * We assume site is live even if it gives a 4xx or 5xx response. 
 * 
 * @param {string} domain 
 * @param {number} timeoutSec 
 * @returns {boolean} 
 */
async function isLive(domain, timeoutSec) {
    const controller = new AbortController();
    const id = setTimeout(() => controller.abort(), timeoutSec * 1000);
    try {
        const res = await fetch(domain, {
            method: 'HEAD',
            headers: {
                'User-Agent': 'Mozilla/5.0 ...',
                'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
                'Accept-Language': 'en-US,en;q=0.5'
            },
            signal: controller.signal,
        });
        clearTimeout(id);
        return res.ok || (res.status >= 100 && res.status < 600);
    } catch {
        clearTimeout(id);
        return false;
    }
}


/**
 * Get page's source code as raw data, no freaking parsing lads!
 * 
 * @param {string} domain 
 * @param {number} timeoutSec
 * @returns {string} 
 */
async function fetchHTML(domain, timeoutSec) {
    const controller = new AbortController();
    const id = setTimeout(() => controller.abort(), timeoutSec * 1000);
    try {
        const res = await fetch(domain, {
            method: 'GET',
            headers: {
                'User-Agent': 'Mozilla/5.0 ...',
                'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
                'Accept-Language': 'en-US,en;q=0.5'
            },
            signal: controller.signal,
        });
        clearTimeout(id);
        if (!res.ok) {
            console.log(`Failed to fetch ${domain}: ${res.status} ${res.statusText}`);
            return '';
        }
        const html = await res.text();
        console.log(`Fetched content from ${domain}: ${html}`);
        return html;
    } catch (error) {
        clearTimeout(id);
        console.log(`Error fetching ${domain}: ${error.message}`);
        return '';
    }
}


/**
 * Checks HTML against fingerprints in fingerprints.json or modified_fingerprints.json 
 * If a match is found then the site might be vulnerable to takeover 
 * but it's not a surefire method without checking DNS records.
 * 
 * @param {string} html 
 * @param {Array} fingerprints
 * @returns {object} 
 */
function checkFingerprints(html, fingerprints) {
    console.log(`Checking fingerprints for content: ${html}`);
    for (const fp of fingerprints) {
        console.log(`Looking for fingerprint: "${fp.fingerprint}"`); 
        if (html.includes(fp.fingerprint)) {
            console.log(`Fingerprint matched for service: ${fp.service_name}`); 
            return { service: fp.service_name, status: 'Vulnerable' };
        }
    }
    console.log(`No fingerprints matched. Status set to Safe.`);
    return { service: '', status: 'Safe' };
}


/**
 * Scans for subdomain takeover, we take domains, timeout, threads and 'live scanner' setting 
 * from user through GUI and then do the real magic here.
 * 
 * @param {string[]} domains 
 * @param {number} timeoutSec 
 * @param {number} concurrency
 * @param {boolean} liveDetector 
 * @param {function(completed, totalCount):void} onProgress - Update progress bar when a domain is fully processed
 * @param {function(id, service, url, status, timestamp):void} onResult - Called when a domain's status is determined
 * @param {object} stoppedRef
 */
async function scanTakeover(domains, timeoutSec, concurrency, liveDetector, onProgress, onResult, stoppedRef) {
    // Dynamically load fingerprints at the start of each scan because fingerprints might change during the time app runs
    const realFingerprints = loadFingerprints();
    if (realFingerprints.length === 0) {
        console.error('No fingerprints available. Aborting scan.');
        return;
    }

    const queue = domains.slice();
    let completed = 0;
    const totalCount = domains.length;

    // Initialize resultId to 1
    let resultId = 1;

    async function worker() {
        while (queue.length > 0) {
            if (stoppedRef.current) break;

            const domain = queue.shift();
            if (!domain) break;

            // Standardize and validate the domain
            const standardized = await standardizeDomain(domain);

            // Skip invalid domains
            if (!standardized) {
                console.log(`Skipped invalid domain: ${domain}`);
                completed++;
                onProgress(completed, totalCount);
                continue;
            }

            let status = 'Not Live';
            let service = '';
            let timestamp = '';

            if (liveDetector) {
                const live = await isLive(standardized, timeoutSec);
                if (stoppedRef.current) break;
                if (!live) {
                    console.log(`Domain not live, skipped from results: ${standardized}`);
                    completed++;
                    onProgress(completed, totalCount);
                    continue;
                } else {
                    // Site is live, fetch HTML and check its source code against fingerprints
                    const html = await fetchHTML(standardized, timeoutSec);
                    if (stoppedRef.current) break;
                    const result = checkFingerprints(html, realFingerprints);
                    service = result.service;
                    status = result.status;
                    timestamp = getFormattedTimestamp();
                }
            } else {
                // Live detection disabled, directly fetch HTML
                // This trash might need an update, it looks similar to above ^
                const html = await fetchHTML(standardized, timeoutSec);
                if (stoppedRef.current) break;
                const result = checkFingerprints(html, realFingerprints);
                service = result.service;
                status = result.status;
                timestamp = getFormattedTimestamp(); 
            }

            completed++;
            onProgress(completed, totalCount);

            // Only send results if status is 'Vulnerable' or 'Safe'
            if (status === 'Vulnerable' || status === 'Safe') {
                onResult(resultId, service, standardized, status, timestamp);
                console.log(`Result Sent: ${standardized} - Status: ${status} - Time: ${timestamp}`);
                resultId++;
            } else {
                console.log(`Status "${status}" not sent for: ${standardized}`);
            }
        }
    }

    // Create an array of worker promises based on the specified concurrency
    const workers = Array.from({ length: concurrency }, () => worker());
    await Promise.all(workers);
}

module.exports = {
    scanTakeover
};
