
async function isLive(domain, timeoutSec, abortControllersRef) {
    const controller = new AbortController();
    abortControllersRef.current.push(controller);
    const id = setTimeout(() => controller.abort(), timeoutSec * 1000);
    try {
      const res = await fetch(domain, {
        method: 'HEAD',
        headers: {
          'User-Agent': 'Mozilla/5.0 ...',
          'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
          'Accept-Language': 'en-US,en;q=0.5'
        },
        signal: controller.signal,
      });
      clearTimeout(id);
      return res.ok || (res.status >= 100 && res.status < 600);
    } catch {
      clearTimeout(id);
      return false;
    } finally {
      abortControllersRef.current = abortControllersRef.current.filter(c => c !== controller);
    }
  }
  
  function processDomains(text) {
    const lines = text
      .split('\n')
      .map(l => l.trim())
      .filter(l => l.length > 0);
    const unique = Array.from(new Set(lines));
    return unique;
  }
  
  function standardizeDomain(domain) {
    let d = domain.trim();
    let url;
    try {
      if (d.match(/^https?:\/\//)) {
        url = new URL(d);
      } else {
        const hasPort = d.includes(':') && !d.includes('//');
        let protocol = 'https://';
        if (hasPort) {
          const port = d.split(':').pop();
          if (port === '80' || port === '8080') {
            protocol = 'http://';
          }
        }
        url = new URL(protocol + d);
      }
  
      if (!url.hostname.startsWith('www.')) {
        url.hostname = 'www.' + url.hostname.replace(/^www\./, '');
      }
  
      if (!url.pathname || url.pathname === '') {
        url.pathname = '/';
      } else if (!url.pathname.endsWith('/')) {
        url.pathname += '/';
      }
  
      return url.toString();
    } catch {
      return d;
    }
  }
  
  async function scanDomains(domains, timeoutSec, concurrency, stoppedRef, abortControllersRef, onProgress, onLiveSite) {
    const queue = domains.slice();
    let completed = 0;
    const totalCount = domains.length;
  
    async function worker() {
      while (queue.length > 0) {
        if (stoppedRef.current) {
          break;
        }
  
        const domain = queue.shift();
        if (!domain) break;
        const standardized = standardizeDomain(domain);
        const live = await isLive(standardized, timeoutSec, abortControllersRef);
        if (stoppedRef.current) {
          break;
        }
        if (live) {
          onLiveSite(standardized);
        }
        completed++;
        onProgress(completed, totalCount);
      }
    }
  
    const workers = Array.from({ length: concurrency }, () => worker());
    await Promise.all(workers);
  }
  
  module.exports = {
    processDomains,
    scanDomains,
  };
  